==> ThbgmExtractor-1.6.3.tar.bz2: ThbgmExtractor-1.6.3/gui/ThbgmExtractor/Smdn.Applications.ThbgmExtractor/ExtendedTrackListView.cs <==
//  
//  Author:
//       smdn <smdn@smdn.jp>
// 
//  Copyright (c) 2004-2014 smdn
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

using System;
using System.ComponentModel;
using System.Drawing;
using System.Linq;
using System.Windows.Forms;

using Smdn.Collections;
using Smdn.Formats;
using Smdn.Formats.Thbgm;
using Smdn.Windows.Forms;
using Smdn.Windows.Forms.ThbgmPlayer;

namespace Smdn.Applications.ThbgmExtractor {
  public class ExtendedTrackListView : TrackListView<ExtendedTrackListViewItem> {
    public event EventHandler TrackEffectPropertyChanged;

    private const int extractProgressSubItemIndex = 2;

    public ExtendedTrackListView()
    {
      Columns.Add(new TrackListViewColumnHeader("OutputFileLength", ThbgmExtractor.GetText("OutputFileLength"), 75));
      OwnerDraw = true;

      extractProgressColumnHeader = new TrackListViewColumnHeader("ExtractProgress", ThbgmExtractor.GetText("ExtractionProgress"), 200);
      lastExtractProgressColumnWidth = extractProgressColumnHeader.Width;
    }

    public void ShowExtractProgressColumn()
    {
      if (Columns.Contains(extractProgressColumnHeader))
        Columns.Remove(extractProgressColumnHeader);

      Columns.Insert(extractProgressSubItemIndex, extractProgressColumnHeader);

      extractProgressColumnHeader.Width = lastExtractProgressColumnWidth;

      UpdateItems();
      UpdateAllTracksItemSubItems();
    }

    public void HideExtractProgressColumn()
    {
      if (Columns.Contains(extractProgressColumnHeader)) {
        lastExtractProgressColumnWidth = extractProgressColumnHeader.Width;

        Columns.Remove(extractProgressColumnHeader);
      }

      foreach (var item in TrackItems) {
        item.ExtractProgress = null;
      }

      UpdateItems();
      UpdateAllTracksItemSubItems();
    }

    protected override string GetAllTracksSubItemText(TrackListViewColumnHeader header)
    {
      switch (header.BindingValueName) {
        case "Title":
          if (Product == null)
            return string.Empty;
          else
            return Product.Title;

        case "ExtractProgress":
          if (extractProgress == null)
            return string.Empty;
          else
            return ThbgmExtractor.GetText("ExtendedTrackListView_ExtractProgress",
                                           extractProgress.TotalCount - (extractProgress.SucceededCount + extractProgress.FailedCount),
                                           extractProgress.SucceededCount,
                                           extractProgress.FailedCount);

        case "OutputFileLength": {
          var totalLength = 0L;

          foreach (var item in TrackItems) {
            if (item.Track != null && item.Track.Checked) // can't refer item.Checked; will cause ArgumentOutOfRangeException on init
              totalLength += item.Track.TrackInfo.StreamFormat.ToAlignedByteCount(item.Track.Effect.TotalLength);
          }

          return string.Format(SIPrefixFormat.CurrentInfo, "{0:f}", totalLength);
        }

        default:
          return base.GetAllTracksSubItemText(header);
      }
    }

    private void UpdateItems()
    {
      foreach (var item in TrackItems) {
        item.InternalUpdateSubItems();
      }
    }

    public void UpdateExtractProgress(ExtractBackgroundWorker.IExtractTrackProgress extractProgress)
    {
      var item = TrackItems.FirstOrDefault(delegate(ExtendedTrackListViewItem i) {
        return (i.Track.TrackInfo == extractProgress.Track.TrackInfo);
      });

      if (item == null)
        return;

      item.ExtractProgress = extractProgress;
      item.InternalUpdateSubItems();
    }

    public void UpdateExtractProgress(ExtractBackgroundWorker.IExtractProgress extractProgress)
    {
      this.extractProgress = extractProgress;

      UpdateAllTracksItemSubItems();
    }

    internal protected virtual void OnTrackEffectPropertyChanged(EventArgs e)
    {
      var ev = this.TrackEffectPropertyChanged;

      if (ev != null)
        ev(this, e);
    }

    protected override ListViewSubItemEditorBase CreateSubItemEditor(TrackListViewItem item, TrackListViewColumnHeader header, ListViewItem.ListViewSubItem subItem)
    {
      switch (header.BindingValueName) {
        case "OutputFileLength":
          return null;

        case "ExtractProgress":
          return null;

        default:
          return base.CreateSubItemEditor(item, header, subItem);
      }
    }

    protected override void OnDrawColumnHeader(DrawListViewColumnHeaderEventArgs e)
    {
      e.DrawDefault = true;

      base.OnDrawColumnHeader(e);
    }

    protected override void OnDrawItem(DrawListViewItemEventArgs e)
    {
      e.DrawDefault = false;

      base.OnDrawItem(e);
    }

    protected override void OnDrawSubItem(DrawListViewSubItemEventArgs e)
    {
      if (e.Header != extractProgressColumnHeader)
        e.DrawDefault = true;

      var item = e.Item as ExtendedTrackListViewItem;

      if (item == null)
        e.DrawDefault = true;

      if (e.DrawDefault) {
        base.OnDrawSubItem(e);
        return;
      }

      /*
       * draw background
       */
      e.DrawBackground();

      /*
       * draw progress
       */
      const int maxProgressWidth = 60;
      var progressWidth = Math.Min(maxProgressWidth, e.Bounds.Width * 6 / 10);
      var progressBounds = e.Bounds;
      var progressRatio = item.ExtractProgress == null ? 0.0 : item.ExtractProgress.ProgressPercent;

      progressBounds.Width = progressWidth;
      progressBounds.Inflate(-1, -1);

      if (ProgressBarRenderer.IsSupported) {
        ProgressBarRenderer.DrawHorizontalBar(e.Graphics, progressBounds);

        progressBounds.Inflate(-2, -2);
        progressBounds.Width = (int)(progressBounds.Width * progressRatio);

        ProgressBarRenderer.DrawHorizontalChunks(e.Graphics, progressBounds);
      }
      else {
        e.Graphics.FillRectangle(SystemBrushes.Highlight, progressBounds);

        progressBounds.Inflate(-1, -1);

        var progressTextBounds = progressBounds;
        var w = (int)(progressBounds.Width * progressRatio);

        progressBounds.X += w;
        progressBounds.Width -= w;

        e.Graphics.FillRectangle(SystemBrushes.Window, progressBounds);

        using (var sf = new StringFormat()) {
          sf.Alignment = StringAlignment.Center;
          sf.LineAlignment = StringAlignment.Center;
          sf.FormatFlags |= StringFormatFlags.NoWrap;

          e.Graphics.DrawString(progressRatio.ToString("P0"), Font, SystemBrushes.WindowText, progressTextBounds, sf);
        }
      }

      /*
       * draw text
       */
      var textBounds = e.Bounds;

      textBounds.X += progressWidth;
      textBounds.Width -= progressWidth;

      using (var sf = new StringFormat()) {
        sf.Alignment = StringAlignment.Near;
        sf.LineAlignment = StringAlignment.Center;
        sf.FormatFlags |= StringFormatFlags.NoWrap;

        Brush fillBrush = null;
        var textBrush = SystemBrushes.WindowText;
        var highlight = Runtime.IsRunningOnMono
          ? (e.ItemState & ListViewItemStates.Selected) == ListViewItemStates.Selected
          : (e.ItemState & ListViewItemStates.Hot) == ListViewItemStates.Hot;

        if (highlight) {
          if (Focused) {
            fillBrush = SystemBrushes.Highlight;
            textBrush = SystemBrushes.HighlightText;
          }
          else {
            fillBrush = SystemBrushes.Control;
          }
        }

        if (fillBrush != null)
          e.Graphics.FillRectangle(fillBrush, textBounds);

        if (item.ExtractProgress != null)
          e.Graphics.DrawString(item.ExtractProgress.StatusText, Font, textBrush, textBounds, sf);
      }
    }

    private TrackListViewColumnHeader extractProgressColumnHeader;
    private int lastExtractProgressColumnWidth;
    private ExtractBackgroundWorker.IExtractProgress extractProgress;
  }
}

==> ThbgmExtractor-1.6.4.tar.bz2: ThbgmExtractor-1.6.4/gui/ThbgmExtractor/Smdn.Applications.ThbgmExtractor/ExtendedTrackListView.cs <==
//  
//  Author:
//       smdn <smdn@smdn.jp>
// 
//  Copyright (c) 2004-2017 smdn
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

using System;
using System.ComponentModel;
using System.Drawing;
using System.Linq;
using System.Windows.Forms;

using Smdn.Collections;
using Smdn.Formats;
using Smdn.Formats.Thbgm;
using Smdn.Windows.Forms;
using Smdn.Windows.Forms.ThbgmPlayer;

namespace Smdn.Applications.ThbgmExtractor {
  public class ExtendedTrackListView : TrackListView<ExtendedTrackListViewItem> {
    public event EventHandler TrackEffectPropertyChanged;

    private const int extractProgressSubItemIndex = 2;

    public ExtendedTrackListView()
    {
      Columns.Add(new TrackListViewColumnHeader("OutputFileLength", ThbgmExtractor.GetText("OutputFileLength"), 75));
      OwnerDraw = true;

      extractProgressColumnHeader = new TrackListViewColumnHeader("ExtractProgress", ThbgmExtractor.GetText("ExtractionProgress"), 200);
      lastExtractProgressColumnWidth = extractProgressColumnHeader.Width;

      timerProgress = new System.Timers.Timer(50);
      timerProgress.Enabled = true;
      timerProgress.AutoReset = true;
      timerProgress.Start();
      timerProgress.Elapsed += delegate {
        progressMarquePercent = (progressMarquePercent + 2) % 100;
      };
    }

    protected override void Dispose(bool disposing)
    {
      if (disposing) {
        if (timerProgress != null) {
          timerProgress.Stop();
          timerProgress.Dispose();
          timerProgress = null;
        }
      }

      base.Dispose(disposing);
    }

    public void ShowExtractProgressColumn()
    {
      if (Columns.Contains(extractProgressColumnHeader))
        Columns.Remove(extractProgressColumnHeader);

      Columns.Insert(extractProgressSubItemIndex, extractProgressColumnHeader);

      extractProgressColumnHeader.Width = lastExtractProgressColumnWidth;

      UpdateItems();
      UpdateAllTracksItemSubItems();
    }

    public void HideExtractProgressColumn()
    {
      if (Columns.Contains(extractProgressColumnHeader)) {
        lastExtractProgressColumnWidth = extractProgressColumnHeader.Width;

        Columns.Remove(extractProgressColumnHeader);
      }

      foreach (var item in TrackItems) {
        item.ExtractProgress = null;
      }

      UpdateItems();
      UpdateAllTracksItemSubItems();
    }

    protected override string GetAllTracksSubItemText(TrackListViewColumnHeader header)
    {
      switch (header.BindingValueName) {
        case "Title":
          if (Product == null)
            return string.Empty;
          else
            return Product.Title;

        case "ExtractProgress":
          if (extractProgress == null)
            return string.Empty;
          else
            return extractProgress.StatusText;

        case "OutputFileLength": {
          var totalLength = 0L;

          foreach (var item in TrackItems) {
            if (item.Track != null && item.Track.Checked) // can't refer item.Checked; will cause ArgumentOutOfRangeException on init
              totalLength += item.Track.TrackInfo.StreamFormat.ToAlignedByteCount(item.Track.Effect.TotalLength);
          }

          return string.Format(SIPrefixFormat.CurrentInfo, "{0:f}", totalLength);
        }

        default:
          return base.GetAllTracksSubItemText(header);
      }
    }

    private void UpdateItems()
    {
      foreach (var item in TrackItems) {
        item.InternalUpdateSubItems();
      }
    }

    public void UpdateExtractProgress(ExtractBackgroundWorker.IExtractTrackProgress extractProgress)
    {
      var item = TrackItems.FirstOrDefault(delegate(ExtendedTrackListViewItem i) {
        return (i.Track.TrackInfo == extractProgress.Track.TrackInfo);
      });

      if (item == null)
        return;

      item.ExtractProgress = extractProgress;
      item.InternalUpdateSubItems();
    }

    public void UpdateExtractProgress(ExtractBackgroundWorker.IExtractProgress extractProgress)
    {
      this.extractProgress = extractProgress;

      UpdateAllTracksItemSubItems();
    }

    internal protected virtual void OnTrackEffectPropertyChanged(EventArgs e)
    {
      var ev = this.TrackEffectPropertyChanged;

      if (ev != null)
        ev(this, e);
    }

    protected override ListViewSubItemEditorBase CreateSubItemEditor(TrackListViewItem item, TrackListViewColumnHeader header, ListViewItem.ListViewSubItem subItem)
    {
      switch (header.BindingValueName) {
        case "OutputFileLength":
          return null;

        case "ExtractProgress":
          return null;

        default:
          return base.CreateSubItemEditor(item, header, subItem);
      }
    }

    protected override void OnDrawColumnHeader(DrawListViewColumnHeaderEventArgs e)
    {
      e.DrawDefault = true;

      base.OnDrawColumnHeader(e);
    }

    protected override void OnDrawItem(DrawListViewItemEventArgs e)
    {
      e.DrawDefault = false;

      base.OnDrawItem(e);
    }

    protected override void OnDrawSubItem(DrawListViewSubItemEventArgs e)
    {
      if (e.Header != extractProgressColumnHeader)
        e.DrawDefault = true;

      ExtractBackgroundWorker.IProgress progress = null;

      if (e.Item == AllTracksItem)
        progress = extractProgress;

      var item = e.Item as ExtendedTrackListViewItem;

      if (item != null)
        progress = item.ExtractProgress;

      if (progress == null || e.DrawDefault) {
        base.OnDrawSubItem(e);
        return;
      }

      /*
       * draw background
       */
      e.DrawBackground();

      /*
       * draw progress
       */
      const int maxProgressWidth = 60;
      var progressWidth = Math.Min(maxProgressWidth, e.Bounds.Width * 6 / 10);
      var progressBounds = e.Bounds;
      var progressRatio = progress.ProgressPercent;
      var drawMarquee = double.IsNaN(progressRatio) || double.IsInfinity(progressRatio);

      progressBounds.Width = progressWidth;
      progressBounds.Inflate(-1, -1);

      var marqueeWidth = progressBounds.Width * 15 / 100; // 15% of progress bar width

      if (ProgressBarRenderer.IsSupported) {
        ProgressBarRenderer.DrawHorizontalBar(e.Graphics, progressBounds);

        progressBounds.Inflate(-2, -2);

        if (drawMarquee) {
          progressBounds.X += (progressBounds.Width - marqueeWidth) * progressMarquePercent / 100;
          progressBounds.Width = marqueeWidth;
        }
        else {
          progressBounds.Width = (int)(progressBounds.Width * progressRatio);
        }

        ProgressBarRenderer.DrawHorizontalChunks(e.Graphics, progressBounds);
      }
      else {
        if (drawMarquee) {
          progressBounds.X += (progressBounds.Width - marqueeWidth) * progressMarquePercent / 100;
          progressBounds.Width = marqueeWidth;

          e.Graphics.FillRectangle(SystemBrushes.Highlight, progressBounds);
        }
        else {
          e.Graphics.FillRectangle(SystemBrushes.Highlight, progressBounds);

          progressBounds.Inflate(-1, -1);

          var progressTextBounds = progressBounds;

          var w = (int)(progressBounds.Width * progressRatio);

          progressBounds.X += w;
          progressBounds.Width -= w;

          e.Graphics.FillRectangle(SystemBrushes.Window, progressBounds);

          using (var sf = new StringFormat()) {
            sf.Alignment = StringAlignment.Center;
            sf.LineAlignment = StringAlignment.Center;
            sf.FormatFlags |= StringFormatFlags.NoWrap;

            e.Graphics.DrawString(progressRatio.ToString("P0"), Font, SystemBrushes.WindowText, progressTextBounds, sf);
          }
        }
      }

      /*
       * draw text
       */
      var textBounds = e.Bounds;

      textBounds.X += progressWidth;
      textBounds.Width -= progressWidth;

      using (var sf = new StringFormat()) {
        sf.Alignment = StringAlignment.Near;
        sf.LineAlignment = StringAlignment.Center;
        sf.FormatFlags |= StringFormatFlags.NoWrap;

        Brush fillBrush = null;
        var textBrush = SystemBrushes.WindowText;
        var highlight = Runtime.IsRunningOnMono
          ? (e.ItemState & ListViewItemStates.Selected) == ListViewItemStates.Selected
          : (e.ItemState & ListViewItemStates.Hot) == ListViewItemStates.Hot;

        if (highlight) {
          if (Focused) {
            fillBrush = SystemBrushes.Highlight;
            textBrush = SystemBrushes.HighlightText;
          }
          else {
            fillBrush = SystemBrushes.Control;
          }
        }

        if (fillBrush != null)
          e.Graphics.FillRectangle(fillBrush, textBounds);

        e.Graphics.DrawString(progress.StatusText, Font, textBrush, textBounds, sf);
      }
    }

    private TrackListViewColumnHeader extractProgressColumnHeader;
    private int lastExtractProgressColumnWidth;
    private ExtractBackgroundWorker.IExtractProgress extractProgress;
    private System.Timers.Timer timerProgress;
    private int progressMarquePercent = 0;
  }
}
